# Partially stolen from https://bitbucket.org/mblum/libgp/src/2537ea7329ef/.ycm_extra_conf.py
import os, glob
import ycm_core

# These are the compilation flags that will be used in case there's no
# compilation database set (by default, one is not set).
# CHANGE THIS LIST OF FLAGS. YES, THIS IS THE DROID YOU HAVE BEEN LOOKING FOR.
flags = [
    '-Wall',
    '-Wextra',
    '-Wno-long-long',
    '-Wno-variadic-macros',
    '-fexceptions',
    # THIS IS IMPORTANT! Without a "-std=<something>" flag, clang won't know which
    # language to use when compiling headers. So it will guess. Badly. So C++
    # headers will be compiled as C headers. You don't want that so ALWAYS specify
    # a "-std=<something>".
    # For a C project, you would set this to something like 'c99' instead of
    # 'c++11'.
    '-std=c++14',
    # ...and the same thing goes for the magic -x option which specifies the
    # language that the files to be compiled are written in. This is mostly
    # relevant for c++ headers.
    # For a C project, you would set this to 'c' instead of 'c++'.
    '-x', 'c++',
    '-isystem', '/usr/local/include',
    '-I', 'include',
    '-I', '.',
    '-I', 'src',
    '-I','/usr/include/qt4',
]

# Set this to the absolute path to the folder (NOT the file!) containing the
# compile_commands.json file to use that instead of 'flags'. See here for
# more details: http://clang.llvm.org/docs/JSONCompilationDatabase.html
#
# Most projects will NOT need to set this to anything; you can just change the
# 'flags' list of compilation flags. Notice that YCM itself uses that approach.
compilation_database_folder = ''

if compilation_database_folder:
  database = ycm_core.CompilationDatabase( compilation_database_folder )
else:
  database = None


def DirectoryOfThisScript():
  return os.path.dirname( os.path.abspath( __file__ ) )


def MakeRelativePathsInFlagsAbsolute( flags, working_directory ):
  if not working_directory:
    return list( flags )
  new_flags = []
  make_next_absolute = False
  path_flags = [ '-isystem', '-I', '-iquote', '--sysroot=' ]
  for flag in flags:
    new_flag = flag

    if make_next_absolute:
      make_next_absolute = False
      if not flag.startswith( '/' ):
        new_flag = os.path.join( working_directory, flag )

    for path_flag in path_flags:
      if flag == path_flag:
        make_next_absolute = True
        break

      if flag.startswith( path_flag ):
        path = flag[ len( path_flag ): ]
        new_flag = path_flag + os.path.join( working_directory, path )
        break

    if new_flag:
      new_flags.append( new_flag )
  return new_flags


def FlagsForFile( filename ):
  if database:
    # Bear in mind that compilation_info.compiler_flags_ does NOT return a
    # python list, but a "list-like" StringVec object
    compilation_info = database.GetCompilationInfoForFile( filename )
    final_flags = MakeRelativePathsInFlagsAbsolute(
      compilation_info.compiler_flags_,
      compilation_info.compiler_working_dir_ )
  else:
    # relative_to = DirectoryOfThisScript()
    relative_to = ProjectRoot(filename)
    final_flags = MakeRelativePathsInFlagsAbsolute( flags, relative_to ) + ["-I", os.path.abspath(os.path.dirname(filename))] + ExtIncludes(relative_to) + BuildIncludes(relative_to, filename)

  return {
    'flags': final_flags,
    'do_cache': True
  }

def ProjectRoot(filename):
    path = os.path.abspath(os.path.dirname(filename))
    while path != os.path.abspath(os.path.dirname(path)) and not (os.path.isdir(os.path.join(path, ".git")) or os.path.isdir(os.path.join(path, "include")) or os.path.isdir(os.path.join(path, "src"))):
        path = os.path.abspath(os.path.dirname(path))
    if not (os.path.isdir(os.path.join(path, "include")) or os.path.isdir(os.path.join(path, "src"))):
        return os.path.abspath(os.path.dirname(filename))
    else:
        return path

def ExtIncludes(path):
    starting_paths = glob.glob(os.path.join(path, "build", "ext-*")) + glob.glob(os.path.join(path, "ext", "*"))
    paths = starting_paths
    for p in starting_paths:
        paths += glob.glob(os.path.join(p, "include")) + glob.glob(os.path.join(p, "*", "include"))
        paths += glob.glob(os.path.join(p, "src", "ext-*")) + glob.glob(os.path.join(p, "src", "ext-*", "include")) + glob.glob(os.path.join(p, "src", "ext-*", "*", "include"))
    return ["-I%s" % p for p in paths]

def BuildIncludes(project_root, filename):
    file_path = os.path.abspath(os.path.dirname(filename))[len(project_root) + 1:]
    return ["-I%s/build/%s" % (project_root, file_path)]
